package org.xbib.net.http.j2html;

import org.xbib.net.Attributes;
import org.xbib.net.ParameterException;
import org.xbib.net.Resource;
import org.xbib.net.http.HttpResponseStatus;
import org.xbib.net.http.server.resource.HtmlTemplateResourceHandler;
import org.xbib.net.http.server.route.HttpRouterContext;
import org.xbib.net.util.ExceptionFormatter;

import java.io.IOException;

import static org.xbib.j2html.TagCreator.body;
import static org.xbib.j2html.TagCreator.code;
import static org.xbib.j2html.TagCreator.div;
import static org.xbib.j2html.TagCreator.h1;
import static org.xbib.j2html.TagCreator.html;
import static org.xbib.j2html.TagCreator.pre;

public class InternalServerErrorHandler extends J2HtmlResourceHandler {

    public InternalServerErrorHandler() {
    }

    @Override
    protected Resource createResource(HttpRouterContext httpRouterContext)
            throws IOException, ParameterException {
        return new InternalServerErrorResource(this, httpRouterContext);
    }

    protected static class InternalServerErrorResource extends J2HtmlResource {

        protected InternalServerErrorResource(HtmlTemplateResourceHandler templateResourceHandler,
                                              HttpRouterContext httpRouterContext)
                throws IOException, ParameterException {
            super(templateResourceHandler, httpRouterContext);
        }

        @Override
        protected HttpResponseStatus getResponseStatus() {
            return HttpResponseStatus.INTERNAL_SERVER_ERROR;
        }

        @Override
        protected String renderHtml(HttpRouterContext httpRouterContext) {
            Attributes attributes = httpRouterContext.getAttributes();
            String message = attributes.get(String.class, "_message");
            Throwable throwable = attributes.get(Throwable.class, "_throwable");
            String stackTrace = ExceptionFormatter.format(throwable);
            return html(body(h1("Internal Server Error"), div(message), pre(code(stackTrace)))).render();
        }
    }
}
