package org.xbib.net.http.j2html;

import org.xbib.net.ParameterException;
import org.xbib.net.buffer.DataBuffer;
import org.xbib.net.http.HttpHeaderValues;
import org.xbib.net.http.HttpResponseStatus;
import org.xbib.net.http.server.HttpRequest;
import org.xbib.net.http.server.resource.HtmlTemplateResource;
import org.xbib.net.http.server.resource.HtmlTemplateResourceHandler;
import org.xbib.net.http.server.route.HttpRouterContext;

import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.util.Objects;
import java.util.logging.Level;
import java.util.logging.Logger;

import static org.xbib.j2html.TagCreator.body;
import static org.xbib.j2html.TagCreator.document;
import static org.xbib.j2html.TagCreator.h1;
import static org.xbib.j2html.TagCreator.html;
import static org.xbib.net.http.HttpHeaderNames.CONTENT_TYPE;

public class J2HtmlResource extends HtmlTemplateResource {

    private static final Logger logger = Logger.getLogger(J2HtmlResource.class.getName());

    protected HttpRequest request;

    public J2HtmlResource(HtmlTemplateResourceHandler templateResourceHandler,
                          HttpRouterContext httpRouterContext) throws IOException, ParameterException {
        super(templateResourceHandler, httpRouterContext);
    }

    @Override
    public void render(HttpRouterContext context)
            throws IOException {
        try {
            this.request = context.getRequest();
            Objects.requireNonNull(responseBuilder);
            DataBuffer dataBuffer = context.getDataBufferFactory().allocateBuffer();
            dataBuffer.write(renderHtml(context), getCharset());
            HttpResponseStatus httpResponseStatus = responseBuilder.getResponseStatus();
            if (httpResponseStatus == null) {
                httpResponseStatus = getResponseStatus();
            }
            context.status(httpResponseStatus)
                    .setHeader("cache-control", "no-cache")
                    .setHeader("content-length", Integer.toString(dataBuffer.writePosition()))
                    .setHeader(CONTENT_TYPE, "text/html; charset=" + getCharset().displayName())
                    .body(dataBuffer)
                    .done();
        } catch (ParameterException e) {
            logger.log(Level.WARNING, e.getMessage(), e);
            context.status(HttpResponseStatus.BAD_REQUEST)
                    .setHeader(CONTENT_TYPE, HttpHeaderValues.TEXT_PLAIN)
                    .body(e.getMessage())
                    .done();
        }
    }

    protected HttpResponseStatus getResponseStatus() {
        return HttpResponseStatus.OK;
    }

    protected Charset getCharset() {
        return StandardCharsets.UTF_8;
    }

    /**
     * By subclassing this handler, this method should be overridden by custom j2html code.
     * The HTML is here is just a greeting as an example.
     *
     * @param context the router context
     * @return the body string fo the HTTP response
     */
    protected String renderHtml(HttpRouterContext context)
        throws IOException, ParameterException {
        return document(html(body(h1("Hello World"))));
    }
}
