package org.xbib.net.http.server.netty.test;

import org.junit.jupiter.api.Test;
import org.xbib.datastructures.api.Builder;
import org.xbib.datastructures.json.tiny.JsonBuilder;
import org.xbib.net.Parameter;
import org.xbib.net.http.HttpHeaders;
import org.xbib.net.http.HttpMethod;
import org.xbib.net.http.server.HttpRequest;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class HttpRequestTest {

    public HttpRequestTest() {
    }

    @Test
    public void testFormAsMultiMap() {
        Parameter parameter = Parameter.builder().domain(Parameter.Domain.FORM)
                .enableDuplicates()
                .add("key", "key1")
                .add("key", "key2")
                .add("key", "key3")
                .add("value", "value1")
                .add("value", "")
                .add("value", "")
                .add("op", "and")
                .add("op", "")
                .add("op", "")
                .build();
        HttpHeaders httpHeaders = new HttpHeaders();
        HttpRequest httpRequest = org.xbib.net.http.server.netty.HttpRequest.builder()
                .setMethod(HttpMethod.POST)
                .setParameter(parameter)
                .setHeaders(httpHeaders)
                .build();
        assertEquals("{key=[key1, key2, key3], value=[value1, , ], op=[and, , ]}",
                httpRequest.asMultiMap().toString());
    }

    @Test
    public void testJsonBodyAsMultiMap() throws IOException {
        // JSON keys overwrite each other
        Builder builder = JsonBuilder.builder()
                .beginMap()
                .field("key", "key1")
                .field("key", "key2")
                .field("key", "key3")
                .field("value", "value1")
                .field("value", "")
                .field("value", "")
                .field("op", "and")
                .field("op", "")
                .field("op", "")
                .endMap();
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.add("content-type", "application/json");
        HttpRequest httpRequest = org.xbib.net.http.server.netty.HttpRequest.builder()
                .setMethod(HttpMethod.POST)
                .setHeaders(httpHeaders)
                .setBody(ByteBuffer.wrap(builder.build().getBytes(StandardCharsets.UTF_8)))
                .build();
        Objects.requireNonNull(httpRequest.getBody());
        assertEquals("{key=[key3], value=[], op=[]}",
                httpRequest.asMultiMap().toString());
    }
}
