package org.xbib.net.http.server.auth;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Map;

import org.xbib.datastructures.tiny.TinyMap;
import org.xbib.net.Attributes;
import org.xbib.net.UserProfile;

public class BaseUserProfile implements UserProfile {

    private String uid;

    private String euid;

    private String name;

    private Attributes attributes;

    private Collection<String> roles;

    private Collection<String> effectiveRoles;

    private Collection<String> permissions;

    private Collection<String> effectivePermissions;

    private boolean refresh;

    public BaseUserProfile() {
        this.attributes = new BaseAttributes();
        this.roles = new ArrayList<>();
        this.effectiveRoles = new ArrayList<>();
        this.permissions = new ArrayList<>();
        this.effectivePermissions = new ArrayList<>();
        this.refresh = false;
    }

    @Override
    public void setUserId(String uid) {
        this.uid = uid;
    }

    @Override
    public String getUserId() {
        return uid;
    }

    @Override
    public void setEffectiveUserId(String euid) {
        this.euid = euid;
    }

    @Override
    public String getEffectiveUserId() {
        return euid;
    }

    @Override
    public void setName(String name) {
        this.name = name;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public void addRole(String role) {
        roles.add(role);
    }

    @Override
    public void setRoles(Collection<String> roles) {
        this.roles = roles;
    }

    @Override
    public Collection<String> getRoles() {
        return roles;
    }

    @Override
    public void addEffectiveRole(String role) {
        effectiveRoles.add(role);
    }

    @Override
    public void setEffectiveRoles(Collection<String> effectiveRoles) {
        this.effectiveRoles = effectiveRoles;
    }

    @Override
    public Collection<String> getEffectiveRoles() {
        return effectiveRoles;
    }

    @Override
    public void addPermission(String permission) {
        permissions.add(permission);
    }

    @Override
    public void setPermissions(Collection<String> permissions) {
        this.permissions = permissions;
    }

    @Override
    public Collection<String> getPermissions() {
        return permissions;
    }

    @Override
    public void addEffectivePermission(String permission) {
        effectivePermissions.add(permission);
    }

    @Override
    public void setEffectivePermissions(Collection<String> effectivePermissions) {
        this.effectivePermissions = effectivePermissions;
    }

    @Override
    public Collection<String> getEffectivePermissions() {
        return effectivePermissions;
    }

    @Override
    public void setAttributes(Attributes attributes) {
        this.attributes = attributes;
    }

    @Override
    public void updateAttribute(String key, Object value) {
        attributes.put(key, value);
    }

    @Override
    public void triggerRefresh(boolean refresh) {
        this.refresh = refresh;
    }

    @Override
    public boolean isRefreshTriggered() {
        return refresh;
    }

    @Override
    public Attributes getAttributes() {
        return attributes;
    }

    @Override
    public Map<String, Object> asMap() {
        TinyMap.Builder<String, Object> builder = TinyMap.builder();
        builder.putIfNotNull(NAME, getName());
        builder.putIfNotNull(USER_ID,  getUserId());
        builder.putIfNotNull(EFFECTIVE_USER_ID, getEffectiveUserId());
        if (getRoles() != null && !getRoles().isEmpty()) {
            builder.put(ROLES, getRoles());
        }
        if (getEffectiveRoles() != null && !getEffectiveRoles().isEmpty()) {
            builder.put(EFFECTIVE_ROLES, getEffectiveRoles());
        }
        if (getPermissions() != null && !getPermissions().isEmpty()) {
            builder.put(PERMISSIONS, getPermissions());
        }
        if (getEffectivePermissions() != null && !getEffectivePermissions().isEmpty()) {
            builder.put(EFFECTIVE_PERMISSIONS, getEffectivePermissions());
        }
        if (getAttributes() != null && !getAttributes().isEmpty()) {
            builder.put(ATTRIBUTES, getAttributes());
        }
        return builder.build();
    }

    @SuppressWarnings("unchecked")
    public static UserProfile fromMap(Map<String, Object> map) {
        BaseUserProfile userProfile = new BaseUserProfile();
        if (map.containsKey(NAME)) {
            userProfile.setName((String) map.get(NAME));
        }
        if (map.containsKey(USER_ID)) {
            String userId = (String) map.get(USER_ID);
            userProfile.setUserId(userId);
        }
        if (map.containsKey(EFFECTIVE_USER_ID)) {
            String eUserId = (String) map.get(EFFECTIVE_USER_ID);
            userProfile.setEffectiveUserId(eUserId);
        }
        if (map.containsKey(ROLES)) {
            userProfile.setRoles((Collection<String>) map.get(ROLES));
        }
        if (map.containsKey(EFFECTIVE_ROLES)) {
            userProfile.setEffectiveRoles((Collection<String>) map.get(EFFECTIVE_ROLES));
        }
        if (map.containsKey(PERMISSIONS)) {
            userProfile.setPermissions((Collection<String>) map.get(PERMISSIONS));
        }
        if (map.containsKey(EFFECTIVE_PERMISSIONS)) {
            userProfile.setEffectivePermissions((Collection<String>) map.get(EFFECTIVE_PERMISSIONS));
        }
        if (map.containsKey(ATTRIBUTES)) {
            userProfile.setAttributes(new BaseAttributes((Map<String, Object>) map.get(ATTRIBUTES)));
        }
        return userProfile;
    }

    @SuppressWarnings("unchecked")
    public static UserProfile fromMap(UserProfile userProfile, Map<String, Object> map) {
        if (map.containsKey(NAME)) {
            userProfile.setName((String) map.get(NAME));
        }
        if (map.containsKey(USER_ID)) {
            userProfile.setUserId((String) map.get(USER_ID));
        }
        if (map.containsKey(EFFECTIVE_USER_ID)) {
            userProfile.setEffectiveUserId((String) map.get(EFFECTIVE_USER_ID));
        }
        if (map.containsKey(ROLES)) {
            userProfile.setRoles((Collection<String>) map.get(ROLES));
        }
        if (map.containsKey(EFFECTIVE_ROLES)) {
            userProfile.setEffectiveRoles((Collection<String>) map.get(EFFECTIVE_ROLES));
        }
        if (map.containsKey(PERMISSIONS)) {
            userProfile.setPermissions((Collection<String>) map.get(PERMISSIONS));
        }
        if (map.containsKey(EFFECTIVE_PERMISSIONS)) {
            userProfile.setEffectivePermissions((Collection<String>) map.get(EFFECTIVE_PERMISSIONS));
        }
        if (map.containsKey(ATTRIBUTES)) {
            if (userProfile.getAttributes() == null || userProfile.getAttributes().isEmpty()) {
                userProfile.setAttributes(new BaseAttributes((Map<String, Object>) map.get(ATTRIBUTES)));
            } else {
                for (Map.Entry<String, Object> entry : ((Map<String, Object>) map.get(ATTRIBUTES)).entrySet()) {
                    userProfile.updateAttribute(entry.getKey(), entry.getValue());
                }
            }
        }
        return userProfile;
    }
}
