package org.xbib.net.http.server.auth;

import org.xbib.net.UserProfile;
import org.xbib.net.http.server.persist.Codec;

import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Properties;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class MemoryPropertiesUserProfileCodec implements Codec<UserProfile> {

    private static final Map<String, Object> store = new HashMap<>();

    private final ReentrantReadWriteLock lock;

    public MemoryPropertiesUserProfileCodec() {
        this.lock = new ReentrantReadWriteLock();
    }

    @Override
    public UserProfile create(String key) throws IOException {
        BaseUserProfile baseUserProfile = new BaseUserProfile();
        baseUserProfile.setUserId(key);
        return baseUserProfile;
    }

    @Override
    public UserProfile read(String key) throws IOException {
        ReentrantReadWriteLock.ReadLock readLock = lock.readLock();
        try {
            readLock.lock();
            Properties properties = new Properties();
            if (store.containsKey(key)) {
                properties.putAll((Map<?, ?>) store.get(key));
            }
            return toUserProfile(key, properties);
        } finally {
            readLock.unlock();
        }
    }

    @Override
    public void write(String key, UserProfile userProfile) throws IOException {
        ReentrantReadWriteLock.WriteLock writeLock = lock.writeLock();
        try {
            writeLock.lock();
            Properties properties = toProperties(userProfile.asMap());
            store.put(key, properties);
        } finally {
            writeLock.unlock();
        }
    }

    private UserProfile toUserProfile(String key, Properties properties) {
        Map<String, Object> map = new LinkedHashMap<>();
        properties.forEach((k, v) -> map.put(k.toString(), v));
        return BaseUserProfile.fromMap(map);
    }

    private Properties toProperties(Map<String, Object> map) {
        Properties properties = new Properties();
        map.forEach((k,v) -> {
            // filter non-null keys and values for properties semantics
            if (k != null && v != null) {
                properties.put(k, v);
            }
        });
        return properties;
    }

    @Override
    public void remove(String key) throws IOException {
        store.remove(key);
    }

    @Override
    public void purge(long expiredAfterSeconds) throws IOException {
        if (expiredAfterSeconds > 0L) {
            for (Map.Entry<String, Object> entry : store.entrySet()) {
                remove(entry.getKey());
            }
        }
    }

    @Override
    public void destroy() {
        store.clear();
    }
}
