package org.xbib.net.http.server.service;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import org.xbib.net.ParameterDefinition;
import org.xbib.net.PathNormalizer;
import org.xbib.net.http.HttpMethod;
import org.xbib.net.http.server.HttpHandler;
import org.xbib.net.http.server.HttpServiceBuilder;
import org.xbib.net.http.server.domain.HttpSecurityDomain;

public class BaseHttpServiceBuilder implements HttpServiceBuilder {

    protected String prefix;

    protected String pathSpec;

    protected Collection<HttpMethod> methods;

    protected Collection<HttpHandler> handlers;

    protected Collection<ParameterDefinition> parameterDefinitions;

    protected HttpSecurityDomain securityDomain;

    protected BaseHttpServiceBuilder() {
        this.prefix = "";
        this.pathSpec = "/**";
        this.methods = new ArrayList<>();
        this.handlers = new ArrayList<>();
        this.securityDomain = null;
    }

    @Override
    public BaseHttpServiceBuilder setPrefix(String prefix) {
        this.prefix = prefix;
        return this;
    }

    @Override
    public BaseHttpServiceBuilder setPath(String pathSpec) {
        if (pathSpec != null) {
            this.pathSpec = PathNormalizer.normalize(pathSpec);
        }
        return this;
    }

    @Override
    public BaseHttpServiceBuilder addMethod(HttpMethod method) {
        this.methods.add(method);
        return this;
    }

    @Override
    public BaseHttpServiceBuilder addHandler(HttpHandler handler) {
        this.handlers.add(handler);
        return this;
    }

    @Override
    public HttpServiceBuilder setParameterDefinition(ParameterDefinition... parameterDefinition) {
        this.parameterDefinitions = Arrays.asList(parameterDefinition);
        return this;
    }

    @Override
    public BaseHttpServiceBuilder setSecurityDomain(HttpSecurityDomain securityDomain) {
        this.securityDomain = securityDomain;
        return this;
    }

    public BaseHttpService build() {
        if (methods.isEmpty()) {
            methods.add(HttpMethod.GET);
        }
        return new BaseHttpService(this);
    }
}
