package org.xbib.net.http.server.test.session;

import java.io.IOException;
import java.nio.file.Paths;
import java.time.Duration;
import java.util.function.Supplier;

import org.junit.jupiter.api.Test;
import org.xbib.net.http.server.persist.Codec;
import org.xbib.net.http.server.session.Session;
import org.xbib.net.http.server.session.FileJsonSessionCodec;
import org.xbib.net.util.RandomUtil;

import static org.junit.jupiter.api.Assertions.assertEquals;

public class JsonSessionTest {

    public JsonSessionTest() {
    }

    @Test
    void testJsonSession() throws IOException {
        Codec<Session> sessionCodec = newSessionCodec();
        Session session = create(sessionCodec, () -> RandomUtil.randomString(16));
        session.put("a", "b");
        sessionCodec.write(session.id(), session);
        Session session1 = sessionCodec.read(session.id());
        assertEquals("b", session1.get("a").toString());
        sessionCodec.destroy();
    }

    private Codec<Session> newSessionCodec() {
        return new FileJsonSessionCodec("SESSION-TEST", null, 1024,
                Duration.ofDays(1), Paths.get("build/session-test"));
    }

    private Session create(Codec<Session> sessionCodec, Supplier<String> sessionIdGenerator) throws IOException {
        return sessionCodec.create(sessionIdGenerator.get());
    }
}
